@@ -31,7 +31,7 @@ class NETWinInfo;
  * using a more high-level interface than the NETWinInfo/NETRootInfo
  * lowlevel classes.
  *
- * Because of limitiations of the way Qt is implemented on Mac OSX, the WId's
+ * Because of limitations of the way Qt is implemented on Mac OSX, the WId's
  * returned by methods in this class are not compatible with those expected
  * by other Qt methods. So while it should be fine to pass WId's retrieved by
  * for example calling the winId method on a QWidget to methods in this class
@@ -44,6 +44,8 @@ class NETWinInfo;
 class KWINDOWSYSTEM_EXPORT KWindowSystem : public QObject, public NET
 {
     Q_OBJECT
+    Q_PROPERTY(bool isPlatformWayland READ isPlatformWayland CONSTANT)
+    Q_PROPERTY(bool isPlatformX11 READ isPlatformX11 CONSTANT)
 
 public:
     /**
@@ -136,6 +138,36 @@ public:
      */
     static void activateWindow(WId win, long time = 0);
 
+    /**
+     * Requests that window @p win is activated.
+     *
+     * There are two ways how to activate a window, by calling
+     * activateWindow() and forceActiveWindow(). Generally,
+     * applications shouldn't make attempts to explicitly activate
+     * their windows, and instead let the user to activate them.
+     * In the special cases where this may be needed, applications
+     * should use activateWindow(). Window manager may consider whether
+     * this request wouldn't result in focus stealing, which
+     * would be obtrusive, and may refuse the request.
+     *
+     * The usage of forceActiveWindow() is meant only for pagers
+     * and similar tools, which represent direct user actions
+     * related to window manipulation.
+     * Except for rare cases, this request will be always honored,
+     * and normal applications are forbidden to use it.
+     *
+     * In case of problems, consult the KWin README in the kdebase
+     * package (kdebase/kwin/README), or ask on the kwin@kde.org
+     * mailing list.
+     *
+     * @param window the window to make active
+     * @param time X server timestamp of the user activity that
+     *    caused this request
+     *
+     * @since 5.89
+     */
+    Q_INVOKABLE static void activateWindow(QWindow *window, long time = 0);
+
     /**
      * Sets window @p win to be the active window. Note that this
      * should be called only in special cases, applications
@@ -607,6 +639,25 @@ public:
      */
     static QPoint constrainViewportRelativePosition(const QPoint &pos);
 
+    /**
+     * Updates the platform-specific startup id, if any.
+     *
+     * This method is to be called when a running application instance
+     * is reused for handling the request to start this application.
+     * A typical use would be in the handler of the KDBusService activation signal.
+     *
+     * For X11, this updates the id for the Startup Notification protocol,
+     * taking the id from QX11Info::nextStartupId(), if not empty.
+     * For Wayland, this updates the token for the XDG Activation protocol,
+     * taking the token from the "XDG_ACTIVATION_TOKEN" environment variable
+     * and then unsetting it, if not empty.
+     *
+     * @param window the main window (needed by X11 platform)
+     *
+     * @since 5.91
+     */
+    static void updateStartupId(QWindow *window);
+
     /**
      * Enum describing the windowing system platform used by the QGuiApplication.
      * @see platform
@@ -626,6 +677,7 @@ public:
          **/
         Wayland,
     };
+    Q_ENUM(Platform);
     /**
      * Returns the Platform used by the QGuiApplication.
      * This method allows to check for the used windowing system in a cheap and reliable way.
@@ -655,21 +707,21 @@ public:
     /**
      * Requests an xdg_activation_v1 token for a specific window.
      *
-     * @p win window in behalf this request is made
-     * @p serial of the event that triggered the request
-     * @p app_id identifier of the application that we are launching
+     * @param win window in behalf this request is made
+     * @param serial of the event that triggered the request
+     * @param app_id identifier of the application that we are launching
      *
-     * @see currentSerial
+     * @see lastInputSerial
      * @since 5.83
      */
-    static void requestXdgActivationToken(QWindow *win, uint32_t serial, const QString &app_id);
+    Q_INVOKABLE static void requestXdgActivationToken(QWindow *win, uint32_t serial, const QString &app_id);
 
     /**
      * Sets the @p token that will be used when activateWindow is called next
      *
      * @since 5.83
      */
-    static void setCurrentXdgActivationToken(const QString &token);
+    Q_INVOKABLE static void setCurrentXdgActivationToken(const QString &token);
 
     /**
      * Offers the seat's current serial
@@ -678,7 +730,7 @@ public:
      *
      * @since 5.83
      */
-    static quint32 lastInputSerial(QWindow *window);
+    Q_INVOKABLE static quint32 lastInputSerial(QWindow *window);
 
 Q_SIGNALS:
 
